/*
Copyright 2009,2010 by Fabian Schwartau
E-Mail: fabian@opencode.eu

This file is part of HM2PC.

HM2PC is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

HM2PC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with HM2PC  If not, see <http://www.gnu.org/licenses/>.
*/

#include "HamegProtocol.h"

void HMSetChannel(Channel channel)
{
	switch(channel)
	{
		case tristate:
			CHSEL_DDR &= ~(1<<CHSEL_PIN);
			CHSEL_PORTOUT &= ~(1<<CHSEL_PIN);
			break;
		case one:
			CHSEL_DDR |= (1<<CHSEL_PIN);
			CHSEL_PORTOUT |=  (1<<CHSEL_PIN);
			break;
		case two:
			CHSEL_DDR |= (1<<CHSEL_PIN);
			CHSEL_PORTOUT &= ~(1<<CHSEL_PIN);
			break;
	}
}


void HMSendScreenshot(char channel)
{
	Channel oldChannel;
	if(!(CHSEL_DDR&(1<<CHSEL_PIN)))
		oldChannel=tristate;
	else
		oldChannel=HMGetChannel();
	RESAZ_DDR   |= (1<<RESAZ_PIN);
	CLAC_DDR    |= (1<<CLAC_PIN);
	SRQ_DDR     |= (1<<SRQ_PIN);

	CLAC_PORT  &= ~(1<<CLAC_PIN);
	SRQ_PORT   |=  (1<<SRQ_PIN);
	RESAZ_PORT   &= ~(1<<RESAZ_PIN);
	RESAZ_PORT   |=  (1<<RESAZ_PIN);
	if(channel==one)  // channel 1, forced
	{
		HMSetChannel(one);
	}
	if(channel==two) // channel 2, forced
	{
		HMSetChannel(two);
	}
	sleep_1ms(1);
	uart_putc(0x31);
	HMReadAndSend();
	HMSetChannel(oldChannel);
	RESAZ_DDR &= ~(1<<RESAZ_PIN);
	CLAC_DDR  &= ~(1<<CLAC_PIN);
	SRQ_DDR   &= ~(1<<SRQ_PIN);
	SRQ_PORT  &= ~(1<<SRQ_PIN);
}

void HMReadAndSend()
{
	unsigned int i;
	for(i=0; i<2048; i++)
	{
		CLAC_PORT   |=  (1<<CLAC_PIN);
		sleep_1us(1);
		uart_putc(DATA_PORT);
		CLAC_PORT  &= ~(1<<CLAC_PIN);
		sleep_1us(1);
	}
}

char HMWaitForChannel(Channel channel)
{
	unsigned char i;
	HMSetChannel(tristate);
	Channel otherChannel=two;
	if(channel==two)
		otherChannel=one;
	for(i=0; i<100; i++) // 10ms timeout
	{
		if(HMGetChannel()==otherChannel)
			break;
		_delay_us(100);
	}
	if(i==100)
		return 0;
	for(i=0; i<100; i++) // 10ms timeout
	{
		if(HMGetChannel()==channel)
		{
			return 1;
		}
		_delay_us(100);
	}
	return 0;
}

Channel HMGetChannel()
{
	if(CHSEL_PORTIN&(1<<CHSEL_PIN))
		return one;
	else
		return two;
}


