/*
Copyright 2009-2011 by Fabian Schwartau
E-Mail: fabian@opencode.eu

This file is part of HM2PC.

HM2PC is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

HM2PC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with HM2PC.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "MainWnd.h"
#include <QtGui/QMessageBox>
#include <QtGui/QFileDialog>
#include <QtCore/QSettings>
#include <fstream>

MainWnd::MainWnd( QWidget* parent /*= 0*/, Qt::WFlags flags /*= 0*/ ) : 
QMainWindow(parent, flags),
renderArea(this)
{
	setupUi(this);
	timerRefresh = new QTimer(this);
	connect(timerRefresh, SIGNAL(timeout()), this, SLOT(onTimerRefresh()));

	statusBar = new QStatusBar(this);
	this->setStatusBar(statusBar);

	screenData1=new char[2048];
	screenData2=new char[2048];
	for(int i=0; i<2048; i++)
	{
		screenData1[i]=0;
		screenData2[i]=0;
	}
	renderArea.setScreenData(NULL, NULL);
	renderArea.show();
	
	readSettings();
}


MainWnd::~MainWnd()
{
	delete[] screenData1;
}

void MainWnd::onButtonOpen()
{
	int error;
	if((error=hameg.openCom(editCOMPort->text(), editBaud->text().toInt()))!=0)
	{
		QMessageBox::information(this, "ERROR", "Failed to open COM port: "+QString::number(error));
		return;
	}
	if(!hameg.checkConnection())
	{
		QMessageBox::information(this, "Warning: Interface not responding", "This interface is not responding!");
	}
	time.start();
	btOpen->setEnabled(false);
	btClose->setEnabled(true);
	btRefresh->setEnabled(true);
	checkAuto->setEnabled(true);
	btSaveTable->setEnabled(true);
	btSaveImage->setEnabled(true);
	if(checkAuto->isChecked())
		timerRefresh->start();
}

void MainWnd::onButtonClose()
{
	btOpen->setEnabled(true);
	btClose->setEnabled(false);
	btRefresh->setEnabled(false);
	btSaveTable->setEnabled(false);
	btSaveImage->setEnabled(false);
	checkAuto->setEnabled(false);
	hameg.closeCom();
	timerRefresh->stop();
}

void MainWnd::onTimerRefresh()
{
	refreshScreen();
}

void MainWnd::resizeEvent(QResizeEvent* event)
{
	int width=event->size().width()-40;
	int height=event->size().height()-190;
	if(width<=0)
		width=1;
	if(height<=0)
		height=1;
	renderArea.setGeometry(20, 150, width, height);
}

void MainWnd::onButtonSaveImage()
{
	QSettings settings("Schwartau App", "Hameg to PC Interface");
	QString fileName=settings.value("imageFileName").toString();
	fileName=QFileDialog::getSaveFileName(this, "Save dialog", fileName, "(*.png);;(*.jpg);;(*.bmp);;(*.tif)");
	if(fileName=="")
		return;
	settings.setValue("imageFileName", fileName);
	int width=0, height=0;
	if(radioImageFull->isChecked())
	{
		width=2048;
		height=256;
	}
	else if(radioImagePort->isChecked())
	{
		width=renderArea.width();
		height=renderArea.height();
	}
	else if(radioImageCustom->isChecked())
	{
		bool isOK;
		width=lineImageWidth->text().toUInt(&isOK);
		if(!isOK)
		{
			width=renderArea.width();
			statusBar->showMessage("This is not a number, correct the size settings! Used view port!", 3000);
		}
		height=lineImageHeight->text().toUInt(&isOK);
		if(!isOK)
		{
			height=renderArea.height();
			statusBar->showMessage("This is not a number, correct the size settings! Used view port!", 3000);
		}
	}
		renderArea.saveToImage(fileName, width, height);
}

void MainWnd::onImageSaveTypeChanged()
{
	if(radioImageFull->isChecked())
	{
		lineImageHeight->setEnabled(false);
		lineImageWidth->setEnabled(false);
	}
	else if(radioImagePort->isChecked())
	{
		lineImageHeight->setEnabled(false);
		lineImageWidth->setEnabled(false);
	}
	else  // Custom
	{
		lineImageHeight->setEnabled(true);
		lineImageWidth->setEnabled(true);
	}
}

void MainWnd::onRefreshModeChanged(bool isChecked)
{
	unsigned int refreshTime;
	bool isOK;
	refreshTime=editRefreshRate->text().toUInt(&isOK);
	if(!isOK)
		QMessageBox::information(this, "Error switching mode", "Failed to switch the mode, the refresh rate does not seem to be a number!");
	if(refreshTime>1000 || refreshTime<=1)
		QMessageBox::information(this, "Error switching mode", "Failed to switch the mode, the refresh rate is to high!\nChoose something less or eaqual 1000");
	if(!isOK || refreshTime>1000 || refreshTime<1)
	{
		checkAuto->setChecked(false);
		btRefresh->setEnabled(true);
		return;
	}

	if(isChecked)
	{
		btRefresh->setEnabled(false);
		timerRefresh->setInterval(1000/refreshTime);
		timerRefresh->start();
	}
	else
	{
		btRefresh->setEnabled(true);
		timerRefresh->stop();
	}
}

void MainWnd::onRefreshRateChanged(QString)
{
	unsigned int refreshTime;
	bool isOK;
	refreshTime=editRefreshRate->text().toUInt(&isOK);
	if(!isOK)
		statusBar->showMessage("The refresh rate does not seem to be a number!");
	else if(refreshTime>1000 || refreshTime<1)
		statusBar->showMessage("The refresh rate is to high!\nChoose something less or equal 1000");
	else
	{
		statusBar->showMessage("Refresh rate changed", 2000);
		timerRefresh->setInterval(1000/refreshTime);
	}
}

void MainWnd::onButtonRefresh()
{
	refreshScreen();
}

int MainWnd::refreshScreen()
{
	int errorCounter=0;
	char *screenData1TmpPtr=NULL;
	char *screenData2TmpPtr=NULL;
	if(checkChannel1->isChecked())
	{
		screenData1TmpPtr=screenData1;
		if(!hameg.getSingleShot(screenData1, '1'))
		{
			QMessageBox::information(this, "Error getting the screenshot of channel 1", "Failed to get the screenshot of channel 1");
			screenData1TmpPtr=NULL;
			errorCounter++;
		}
	}
	if(checkChannel2->isChecked())
	{
		screenData2TmpPtr=screenData2;
		if(!hameg.getSingleShot(screenData2, '2'))
		{
			QMessageBox::information(this, "Error getting the screenshot of channel 2", "Failed to get the screenshot of channel 2");
			screenData2TmpPtr=NULL;
			errorCounter++;
		}
	}
	renderArea.setScreenData(screenData1TmpPtr, screenData2TmpPtr);
	if(errorCounter==2)
		return 0;
	return 1;
}

void MainWnd::onCheckChannel()
{
	char *screenData1TmpPtr=NULL;
	char *screenData2TmpPtr=NULL;
	if(checkChannel1->isChecked())
	{
		screenData1TmpPtr=screenData1;
	}
	if(checkChannel2->isChecked())
	{
		screenData2TmpPtr=screenData2;
	}
	renderArea.setScreenData(screenData1TmpPtr, screenData2TmpPtr);
}

void MainWnd::readSettings()
{
	QSettings settings("Schwartau App", "Hameg to PC Interface");
    restoreGeometry(settings.value("geometry").toByteArray());
    restoreState(settings.value("windowState").toByteArray());
    QString baud=settings.value("baud").toString();
    if(baud.isEmpty())
    	baud=tr("1000000");
    editBaud->setText(baud);
    
    QString port=settings.value("port").toString();
    if(port.isEmpty())
    	port="/dev/ttyUSB0";
    editCOMPort->setText(port);
    radioImageFull->setChecked(settings.value("radioImageFull").toBool());
    radioImagePort->setChecked(settings.value("radioImagePort").toBool());
    radioImageCustom->setChecked(settings.value("radioImageCustom").toBool());
    if(radioImageCustom->isChecked())
    {
    	lineImageWidth->setEnabled(true);
    	lineImageHeight->setEnabled(true);
    }
    else
    {
    	lineImageWidth->setEnabled(false);
    	lineImageHeight->setEnabled(false);
    }
    QString imageWidth=settings.value("imageWidth").toString();
    if(imageWidth.isEmpty())
    	imageWidth=tr("2048");
    lineImageWidth->setText(imageWidth);
    QString imageHeight=settings.value("imageHeight").toString();
    if(imageHeight.isEmpty())
    	imageHeight=tr("246");
    lineImageHeight->setText(imageHeight);

    QString refreshRate=settings.value("refreshRate").toString();
    if(refreshRate.isEmpty())
    	refreshRate=tr("5");
    editRefreshRate->setText(refreshRate);
    
    checkChannel1->setChecked(settings.value("channel1").toBool());
    checkChannel2->setChecked(settings.value("channel2").toBool());
}

void MainWnd::closeEvent(QCloseEvent *event)
{
    QSettings settings("Schwartau App", "Hameg to PC Interface");
    settings.setValue("geometry", saveGeometry());
    settings.setValue("windowState", saveState());
    settings.setValue("baud", editBaud->text());
    settings.setValue("port", editCOMPort->text());
    settings.setValue("radioImageFull", radioImageFull->isChecked());
    settings.setValue("radioImagePort", radioImagePort->isChecked());
    settings.setValue("radioImageCustom", radioImageCustom->isChecked());
    settings.setValue("imageWidth", lineImageWidth->text());
    settings.setValue("imageHeight", lineImageHeight->text());
    settings.setValue("refreshRate", editRefreshRate->text());
    
    settings.setValue("channel1", checkChannel1->isChecked());
    settings.setValue("channel2", checkChannel2->isChecked());
    QMainWindow::closeEvent(event);
}

void MainWnd::onButtonSaveTable()
{
	QSettings settings("Schwartau App", "Hameg to PC Interface");
	QString fileName=settings.value("tableFileName").toString();
	fileName=QFileDialog::getSaveFileName(this, "Save dialog", fileName, "(*.cvs)");
		if(fileName=="")
			return;
	settings.setValue("tableFileName", fileName);
	bool chan1=checkChannel1->isChecked() && screenData1!=NULL;
	bool chan2=checkChannel2->isChecked() && screenData2!=NULL;
	ofstream output(fileName.toStdString().c_str());
	if(!output.good())
	{
		QMessageBox::information(this, "Failed to save file!", "Unable to save the file, maybe you don't have permissions to write this file");
		return;
	}
	output<<"Sample";
	if(chan1)
		output<<";Channel 1";
	if(chan2)
		output<<";Channel 2";
	output<<endl;
	for(int i=0; i<2048; i++)
	{
		output<<i;
		if(chan1)
			output<<";"<<(unsigned int)(unsigned char)screenData1[i];
		if(chan2)
			output<<";"<<(unsigned int)(unsigned char)screenData2[i];
		output<<endl;
	}
	output.close();
}
