/*
Copyright 2009-2011 by Fabian Schwartau
E-Mail: fabian@opencode.eu

This file is part of HM2PC.

HM2PC is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

HM2PC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with HM2PC.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "RenderArea.h"
#include <QtGui/QMessageBox>

#define LINE_NUMBER_HOR   11
#define LINE_NUMBER_VER   11
#define LINE_FIRST_HOR    18
#define LINE_LAST_HOR     247
#define LINE_FIRST_VER    0
#define LINE_LAST_VER     1990

RenderArea::RenderArea(QWidget* parent) :
//QWidget(parent),
QFrame(parent),
screenData1(NULL),
screenData2(NULL)
{
	this->setGeometry(20,150,200,200);
	/*setFrameStyle(QFrame::Box | QFrame::Raised);
	setLineWidth(2);
	setMidLineWidth(2);
	setFrameShadow(QFrame::Plain);
	setFrameShape(QFrame::Box);*/
}

RenderArea::~RenderArea()
{
	
}

void RenderArea::paintEvent(QPaintEvent*)
{
	repaintLines(this);
	QPainter painter(this);
	painter.setPen(Qt::gray);
	painter.setBrush(Qt::NoBrush);
	painter.drawRect(QRect(0, 0, width() - 1, height() - 1));
}

void RenderArea::repaintLines(QPaintDevice* device)
{
	QPainter painter(device);
	painter.setRenderHint(QPainter::Antialiasing, false);
	painter.setPen(Qt::white);
	painter.fillRect(0, 0, device->width(), device->height(), Qt::white);
	
	painter.setPen(Qt::gray);
	// draw the vertical lines on the screen
	for(int i=0; i<LINE_NUMBER_VER; i++)
	{
		painter.drawLine(
				device->width()*(LINE_FIRST_VER+i*(LINE_LAST_VER-LINE_FIRST_VER)/(LINE_NUMBER_VER-1))/2048, device->height()*LINE_FIRST_HOR/256,
				device->width()*(LINE_FIRST_VER+i*(LINE_LAST_VER-LINE_FIRST_VER)/(LINE_NUMBER_VER-1))/2048, device->height()*LINE_LAST_HOR/256
				);
	}
	
	// draw the horizontal lines on the screen
	for(int i=0; i<LINE_NUMBER_HOR; i++)
	{
		painter.drawLine(
				device->width()*LINE_FIRST_VER/2048, device->height()*(LINE_FIRST_HOR+i*(LINE_LAST_HOR-LINE_FIRST_HOR)/(LINE_NUMBER_HOR-1))/256,
				device->width()*LINE_LAST_VER/2048, device->height()*(LINE_FIRST_HOR+i*(LINE_LAST_HOR-LINE_FIRST_HOR)/(LINE_NUMBER_HOR-1))/256
				);
	}
	
	// draw the 1/5 lines on the horizontal axis
	for(int i=0; i<(LINE_NUMBER_VER-1)*5; i++)
	{
		painter.setPen(Qt::gray);
		int x=device->width()*(LINE_FIRST_VER+i*(LINE_LAST_VER-LINE_FIRST_VER)/(LINE_NUMBER_HOR-1)/5)/2048;
		painter.drawLine(
				x, device->height()*(LINE_FIRST_HOR+(LINE_LAST_HOR-LINE_FIRST_HOR)/2-3)/256,
				x, device->height()*(LINE_FIRST_HOR+(LINE_LAST_HOR-LINE_FIRST_HOR)/2+3)/256
				);
		//painter.setPen(Qt::black);
		painter.drawPoint(x, device->height()*(LINE_FIRST_HOR+(LINE_LAST_HOR-LINE_FIRST_HOR)*17/20)/256);
		painter.drawPoint(x, device->height()*(LINE_FIRST_HOR+(LINE_LAST_HOR-LINE_FIRST_HOR)*3/20)/256);
	}
	painter.setPen(Qt::gray);

	// draw the 1/5 lines on the vertical axis
	for(int i=0; i<(LINE_NUMBER_HOR-1)*5; i++)
	{
		painter.drawLine(
				device->width()*(LINE_FIRST_VER+(LINE_LAST_VER-LINE_FIRST_VER)/2-12)/2048, device->height()*(LINE_FIRST_HOR+i*(LINE_LAST_HOR-LINE_FIRST_HOR)/(LINE_NUMBER_VER-1)/5)/256, 
				device->width()*(LINE_FIRST_VER+(LINE_LAST_VER-LINE_FIRST_VER)/2+12)/2048, device->height()*(LINE_FIRST_HOR+i*(LINE_LAST_HOR-LINE_FIRST_HOR)/(LINE_NUMBER_VER-1)/5)/256
				);
	}
	
	// First screen
	if(screenData1)
	{
		painter.setPen(Qt::red);
		for(int i=1; i<2048;i++)
		{
			unsigned int value1=(unsigned int)((unsigned char)screenData1[i-1]);
			value1=device->height()-value1*device->height()/255;
			unsigned int value2=(unsigned int)((unsigned char)screenData1[i]);
			value2=device->height()-value2*device->height()/255;
			painter.drawLine(device->width()*(i-1)/2048, value1, device->width()*(i)/2048, value2);
		}
	}

	// Second screen
	if(screenData2)
	{
		painter.setPen(Qt::green);
		for(int i=1; i<2048;i++)
		{
			unsigned int value1=(unsigned int)((unsigned char)screenData2[i-1]);
			value1=device->height()-value1*device->height()/255;
			unsigned int value2=(unsigned int)((unsigned char)screenData2[i]);
			value2=device->height()-value2*device->height()/255;
			painter.drawLine(device->width()*(i-1)/2048, value1, device->width()*(i)/2048, value2);
		}
	}
}

void RenderArea::setScreenData(char* screenData1, char* screenData2)
{
	this->screenData1=screenData1;
	this->screenData2=screenData2;
	this->repaint();
}

void RenderArea::saveToImage(QString fileName, int width, int height)
{
	//QPixmap pixmap= QPixmap::grabWindow(winId());   // FIXME: this is not very porable
	//pixmap.save(fileName);

	QImage image(width, height, QImage::Format_RGB32);
	repaintLines(&image);
	if(!image.save(fileName))
		QMessageBox::information(this, "Saving image failed", "Failed to save image, maybe you don't have permission to write this file.");
}




